var global = {
  sessionKey: false,
  uid: false,
  authToken: false,
  windows: {
    generic: false,
    notification: false,
    history: false,
    login: false,
    help: false,
    about: false,
    loading: false
  }
}

//replaceVars
//-----
//text: full string to replace vars
//a: associative array, keys are replaced with their value
function replaceVars(text, a) {
  for(var i in a) {
    text = text.replace('@@' + i + '@@', a[i]);
  }
  return text;
};

//showNotification
//-----
//props:
//  messages: main message text
//  file: html template
//  where: location of the popup window
//  width: window width
//  height: window height
//  windowRef: global window reference
//  alwaysInFront: display the window on top of anything else
//  doActivate: run the activate() method for a window, brings it to the front
//  title: optional window title
//  closeEvent: function to run on window close
function showNotification(props) {
  if(!props) props = {};
  var userProps = {
    messages: props.messages || '[messages]',
    file: props.file || 'notification.html',
    where: props.where || 'center',
    width: props.width || 400,
    height: props.height || 400,
    windowRef: props.windowRef || 'generic',
    alwaysInFront: props.alwaysInFront || false,
    doActivate: props.doActivate || false,
    title: props.title || false,
    closeEvent: props.closeEvent || false
  }

  var options = new air.NativeWindowInitOptions(); 
  options.systemChrome = air.NativeWindowSystemChrome.NONE; 
  options.type = air.NativeWindowType.LIGHTWEIGHT;
  options.transparent = true;
  
  var rectX = 0;
  var rectY = 0;
  switch(userProps.where) {
    case 'topright':
      rectX = air.Capabilities.screenResolutionX - userProps.width - 75;
      rectY = 50;
      break;
    case 'bottomright':
      rectX = air.Capabilities.screenResolutionX - userProps.width - 20;
      rectY = air.Capabilities.screenResolutionY - userProps.height - 20;
      break;
    case 'center':
    default:
      rectX = (air.Capabilities.screenResolutionX / 2) - (userProps.width / 2);
      rectY = (air.Capabilities.screenResolutionY / 2) - (userProps.height / 2);
  }
  
  global.windows[userProps.windowRef] = new runtime.flash.display.NativeWindow(options);
  global.windows[userProps.windowRef].bounds = new air.Rectangle(rectX, rectY, userProps.width, userProps.height);
  global.windows[userProps.windowRef].alwaysInFront = userProps.alwaysInFront;
  if(userProps.title) global.windows[userProps.windowRef].title = userProps.title;
  
  var loader = new air.URLLoader();
  var request = new air.URLRequest(userProps.file);
  loader.load(request);
  loader.addEventListener(air.Event.COMPLETE, function(event) {
    var l = air.URLLoader(event.target);
    var htmlLoader = new air.HTMLLoader();
    htmlLoader.width = userProps.width;
    htmlLoader.height = userProps.height;
    htmlLoader.paintsDefaultBackground = false;
    global.windows[userProps.windowRef].stage.align = 'TL';
    global.windows[userProps.windowRef].stage.scaleMode = 'noScale';
    global.windows[userProps.windowRef].stage.addChild(htmlLoader);
    htmlLoader.loadString(replaceVars(l.data, {
      messages: userProps.messages
    }));
    global.windows[userProps.windowRef].visible = true;
    if(userProps.doActivate) global.windows[userProps.windowRef].activate();
    
    htmlLoader.addEventListener(air.Event.LOCATION_CHANGE, function(event) {
      if(htmlLoader.location.indexOf('closewindow.html') >= 0) {
        global.windows[userProps.windowRef].close();
        if(userProps.closeEvent) userProps.closeEvent();
      }      
    });
  });
};

//showLogin
//-----
//[no args]
function showLogin() {
  var width = 646;
  var height = 500;
  var options = new air.NativeWindowInitOptions();
  var rectX = (air.Capabilities.screenResolutionX / 2) - (width / 2);
  var rectY = (air.Capabilities.screenResolutionY / 2) - (height / 2);
  
  global.windows.login = new runtime.flash.display.NativeWindow(options);
  global.windows.login.bounds = new air.Rectangle(rectX, rectY, width, height);
  global.windows.login.title = 'flair login';
  
  var htmlLoader = new air.HTMLLoader();
  htmlLoader.width = width;
  htmlLoader.height = height;
  global.windows.login.stage.align = 'TL';
  global.windows.login.stage.scaleMode = 'noScale';
  global.windows.login.stage.addChild(htmlLoader);
  
  htmlLoader.addEventListener(air.Event.LOCATION_CHANGE, function(event) {
    //close loading window
    if(htmlLoader.location.indexOf('login.php') >= 0) {
      global.windows.loading.close();
    }
    
    //close login
    if(htmlLoader.location.indexOf('desktopapp.php') >= 0) {
      global.windows.login.close();
      getSessionKey();
      displayNotificationsTimer.start();
      storeNotificationsTimer.start();
      showNotification({
        file: 'loggedin_' + (air.NativeApplication.supportsSystemTrayIcon ? 'win' : 'mac') + '.html',
        height: 320
      });
      
      var historyTimer = new air.Timer(10000, 1);
      historyTimer.addEventListener(air.TimerEvent.TIMER, function() {
        displayHistory();
      });
      historyTimer.start();
    }
  });
  
  htmlLoader.load(new air.URLRequest('https://login.facebook.com/login.php?api_key=4af5249d16b0b979ac9d77dadfabeb77&v=1.0&popup=1&hide_checkbox=1&auth_token=' + readEncryptedStore('authToken')));
  global.windows.login.visible = true;
  global.windows.login.alwaysInFront = true;
  global.windows.login.activate();
  global.windows.login.alwaysInFront = false;
  htmlLoader.window.focus();
};

//showLoading
//-----
//[no args]
function showLoading() {
  showNotification({
    file: 'loading.html',
    height: 250,
    windowRef: 'loading',
    alwaysInFront: true
  });
};

//aboutWindow
//-----
//[no args]
function aboutWindow() {  
  showNotification({
    file: 'about.html',
    height: 300,
    windowRef: 'about'
  });
};

//closeWindow
//-----
//w: window reference to close
function closeWindow(w) {
  dojo.fadeOut({node: 'box', duration: 1000}).play();
    
  var wTimer = new air.Timer(1001, 1);
  wTimer.addEventListener(air.TimerEvent.TIMER, function() {
    location.href = 'closewindow.html';
  });
  wTimer.start(); 
};

//helpWindow
//-----
//[no args]
function helpWindow(c) {
  showNotification({
    file: 'help.html',
    where: 'center',
    width: 500,
    height: 600,
    windowRef: 'help',
    closeEvent: c
  });
};

//facebookWindow
//-----
//[no args]
function facebookWindow() {
  var uid = readEncryptedStore('uid');
  if(uid <= 0 || (uid + '').length <= 0) {
    if(global.windows.login && global.windows.login.closed) {
      showLogin();
    }
    return false;
  }
  air.navigateToURL(new air.URLRequest('http://www.facebook.com/profile.php?id=' + uid));
};

//SQLErrorHandler
//-----
//event: event passed from the SQL call
function SQLErrorHandler(event) {
  air.trace('db error: ', event.error.message);
  air.trace('details: ', event.error.details);
}

//connectToDB
//-----
//[no args]
function connectToDB() {
  //create or open the db
  var conn = new air.SQLConnection();
  conn.addEventListener(air.SQLEvent.OPEN, function(event) {
    //air.trace('database created');
  });
  conn.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
  var db = air.File.applicationStorageDirectory.resolvePath('fb.db');
  conn.open(db);
  
  //add tables if needed
  var createStmt = new air.SQLStatement();
  createStmt.sqlConnection = conn;
  
  createStmt.text = "CREATE TABLE IF NOT EXISTS notifications (id INTEGER PRIMARY KEY AUTOINCREMENT, type TEXT, notification TEXT, xmltimestamp INTEGER, status INTEGER, inserted INTEGER, uid INTEGER)";
  createStmt.execute();
  
  return conn;
};

//displayNewNotifications
//-----
//props:
//  conn: database connection or autoconnect
function displayNewNotifications(props) {
  if(!props) props = {};
  var userProps = {
    conn: props.conn || connectToDB()
  }

  if(userProps.conn) {
    var selectStmt = new air.SQLStatement();
    selectStmt.sqlConnection = userProps.conn;
    selectStmt.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND status = 1";
    
    //errors
    selectStmt.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
    
    //display notification window
    selectStmt.addEventListener(air.SQLEvent.RESULT, function(event) {
      var msgs = '';
      var result = selectStmt.getResult();
      if(result == null || result.data == null) return false;
      
      for(var i = 0; i < result.data.length; i++) {
        var row = result.data[i];
        
        var msg = '';
        var dp = new DOMParser();
        var xml = dp.parseFromString('<notification>' + row.notification + '</notification>', 'text/xml');
        
        switch(row.type) {
          case 'pokes':
            var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
            msg = '<strong>' + newCount + '</strong> new poke' + (newCount > 1 ? 's' : '');
            break;
          case 'shares':
            var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
            msg = '<strong>' + newCount + '</strong> new share' + (newCount > 1 ? 's' : '');
            break;
          case 'friend_requests':
            var uids = xml.documentElement.getElementsByTagName('uid');
            msg = '<strong>' + uids.length + '</strong> new friend request' + (uids.length > 1 ? 's' : '');
            break;
          case 'group_invites':
          
            break;
          case 'event_invites':
          
            break;
          case 'messages':
          default:
            var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
            msg = '<strong>' + newCount + '</strong> new message' + (newCount > 1 ? 's' : '');
            break;
        }
        
        msgs += '<p>' + msg + '</p>';
        
      }
      
      showNotification({
        messages: msgs,
        where: 'topright',
        windowRef: 'notification',
        alwaysInFront: true
      });
    });
    
    selectStmt.execute();
    
    //mark as displayed in the db
    var updateStmt = new air.SQLStatement();
    updateStmt.sqlConnection = userProps.conn;
    updateStmt.text = "UPDATE notifications SET status = 0 WHERE uid = " + readEncryptedStore('uid') + " AND status = 1";
    updateStmt.execute();
  }
};

//displayHistory
//-----
//props:
//  conn: database connection or autoconnect
function displayHistory(props) {
  if(!props) props = {};
  var userProps = {
    conn: props.conn || connectToDB()
  }
  
  //make sure user is logged in
  var uid = readEncryptedStore('uid');
  if(uid <= 0 || (uid + '').length <= 0) {
    if(global.windows.login && global.windows.login.closed) {
      showLogin();
    }
    return false;
  }
  
  //history already open
  if(global.windows.history && !global.windows.history.closed) {
    global.windows.history.alwaysInFront = true; //bring the window above anything else
    global.windows.history.activate();
    global.windows.history.alwaysInFront = false; //allow other apps to once again be brought in front
    return true;
  }

  if(userProps.conn) {  
    var selectStmt = new air.SQLStatement();
    selectStmt.sqlConnection = userProps.conn;
    selectStmt.text = "SELECT * FROM notifications WHERE uid = " + uid + " ORDER BY inserted DESC";
    
    //errors
    selectStmt.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
    
    //display notification window
    selectStmt.addEventListener(air.SQLEvent.RESULT, function(event) {
      var msgs = '';
      var dateLine = '';
      var result = selectStmt.getResult();
      if(result == null || result.data == null) {
        msgs = 'You don\'t have any notifications yet!';
      }
      else {
        for(var i = 0; i < result.data.length; i++) {
          var row = result.data[i];
          
          var msg = '';
          var dp = new DOMParser();
          var xml = dp.parseFromString('<notification>' + row.notification + '</notification>', 'text/xml');
          
          var timestamp = new Date();
          timestamp.setTime(parseInt(row.inserted) * 1000);
          var ampm = 'am';
          var hours = timestamp.getHours();
          if(hours == 0) {
            hours = 12;
          }
          else if(hours == 12) {
            ampm = 'pm';
          }
          else if(hours > 12) {
            hours -= 12;
            ampm = 'pm';
          }
          if(hours < 10) {
            hours = '0' + hours;
          }
          var mins = timestamp.getMinutes();
          if(mins < 10) mins = '0' + mins;
          
          //header
          var currentDateLine = getDayOfWeek(timestamp) + ' - ' + getMonth(timestamp) + ' ' + timestamp.getDate() + ', ' + timestamp.getFullYear();
          if(currentDateLine != dateLine) {
            msgs += '<div class="dateSeparator">' + currentDateLine + '</div>';
            dateLine = currentDateLine;
          }          
          
          msg += '<div style="float: left;"><span class="time">' + hours + ':' + mins + ampm + '</span> <span class="dot">&rArr;</span></div>';
          
          switch(row.type) {
            case 'pokes':
              var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
              msg += '<strong>' + newCount + '</strong> new poke' + (newCount > 1 ? 's' : '');
              break;
            case 'shares':
              var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
              msg += '<strong>' + newCount + '</strong> new share' + (newCount > 1 ? 's' : '');
              break;
            case 'friend_requests':
              var uids = xml.documentElement.getElementsByTagName('uid');
              msg += '<div dojoType="dijit.TitlePane" open="false" id="friend' + row.id + '" title="<a style=\'display: block;\' class=\'friendExpand\' href=\'#\' onclick=\'getFriendsDetails(' + row.id + ')\'><strong>' + uids.length + '</strong> new friend request' + (uids.length > 1 ? 's' : '') + ' <span class=\'dot\'>+</span></a>"><span class="time">loading...</span></div>';
              break;
            case 'group_invites':
            
              break;
            case 'event_invites':
            
              break;
            case 'messages':
            default:
              var newCount = xml.documentElement.getElementsByTagName('unread')[0].firstChild.nodeValue;
              msg += '<strong>' + newCount + '</strong> new message' + (newCount > 1 ? 's' : '');
              break;
          }
    
          msgs += '<p style="margin-bottom: 5px;">' + msg + '</p>';
        }
      }
      
      showNotification({
        messages: msgs,
        where: 'center',
        width: 500,
        height: 600,
        file: 'history.html',
        windowRef: 'history'
      });
    });
    
    selectStmt.execute();
  }
};

//callFacebook
//-----
//props:
//  params: associative array or params to send to the API
//  method: API method
//  onComplete: function to run on complete of the API call
function callFacebook(props) {
  if(!props) props = {};
  var userProps = {
    params: props.params || {},
    method: props.method || false,
    onComplete: props.onComplete || false
  }
  
  if(!userProps.method) return false;
  
  var secret = '01a02f7e8e0112babf2dc42518e19b19';
  var loader = new air.URLLoader(); 
  var request = new air.URLRequest('https://api.facebook.com/restserver.php'); 
    
  var params = [];
  params.push('api_key=4af5249d16b0b979ac9d77dadfabeb77');
  params.push('method=' + userProps.method);
  params.push('v=1.0');
  
  //add session key
  if(readEncryptedStore('sessionKey')) {
    params.push('session_key=' + readEncryptedStore('sessionKey'));
    params.push('call_id=' + (new Date()).getTime()); //unique number for every call
    secret = readEncryptedStore('secret');
  }
  
  //add additional user props
  for(var i in userProps.params) {
    params.push(i + '=' + userProps.params[i]);
  }
  
  //generate sig
  params.sort();
  var sig = md5(params.join('') + secret);
  params.push('sig=' + sig);
  
  request.data = params.join('&');
  request.method = air.URLRequestMethod.POST;
  
  if(userProps.onComplete) {
    loader.addEventListener(air.Event.COMPLETE, userProps.onComplete);
  }
  
  try { 
    loader.load(request); 
  } catch(error) { 
    air.trace('api call failed: ' + userProps.method); 
  }

};

//storeNewNotifications
//-----
//[no args]
function storeNewNotifications() {
  if(!readEncryptedStore('sessionKey')) {  
    air.trace('storeNewNotifications failed, no session key yet');
    return false;
  }
  
  callFacebook({
    method: 'notifications.get',
    onComplete: function(event) {
      var loader = air.URLLoader(event.target);
      loader.data = loader.data.replace(' list="true">', '>');
      loader.data = loader.data.replace(' list="false">', '>');
      air.trace('notifications.get: ' + loader.data);
      
      var createStmt = new air.SQLStatement();
      createStmt.sqlConnection = connectToDB();
      
      //messages
      var messagesXML = getXMLValue(loader.data, 'messages');
      if(messagesXML.indexOf('unread>0') < 0) {
        var mostRecent = getXMLValue(messagesXML, 'most_recent');
        var selectStmt1 = new air.SQLStatement();
        selectStmt1.sqlConnection = connectToDB();
        selectStmt1.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'messages' AND xmltimestamp >= " + mostRecent;
        selectStmt1.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt1.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt1.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('messages', '" + messagesXML + "', 1, " + unix_timestamp() + ", " + mostRecent + ", " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted messages');
          }
        });
        selectStmt1.execute();
      }
      
      //pokes
      var pokesXML = getXMLValue(loader.data, 'pokes');
      if(pokesXML.indexOf('unread>0') < 0) {
        var mostRecent = getXMLValue(pokesXML, 'most_recent');
        var selectStmt2 = new air.SQLStatement();
        selectStmt2.sqlConnection = connectToDB();
        selectStmt2.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'pokes' AND xmltimestamp >= " + mostRecent;
        selectStmt2.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt2.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt2.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('pokes', '" + pokesXML + "', 1, " + unix_timestamp() + ", " + mostRecent + ", " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted pokes');
          }
        });
        selectStmt2.execute();
      }
      
      //shares
      var sharesXML = getXMLValue(loader.data, 'shares');
      if(sharesXML.indexOf('unread>0') < 0) {
        var mostRecent = getXMLValue(pokesXML, 'most_recent');
        var selectStmt3 = new air.SQLStatement();
        selectStmt3.sqlConnection = connectToDB();
        selectStmt3.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'pokes' AND xmltimestamp >= " + mostRecent;
        selectStmt3.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt3.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt3.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('shares', '" + sharesXML + "', 1, " + unix_timestamp() + ", " + mostRecent + ", " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted shares');
          }
        });
        selectStmt3.execute();
      }
      
      //friend requests
      var friendRequestsXML = getXMLValue(loader.data, 'friend_requests');
      if(friendRequestsXML) {
        var selectStmt4 = new air.SQLStatement();
        selectStmt4.sqlConnection = connectToDB();
        selectStmt4.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'friend_requests' AND notification = '" + friendRequestsXML + "'";
        selectStmt4.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt4.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt4.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('friend_requests', '" + friendRequestsXML + "', 1, " + unix_timestamp() + ", 0, " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted friend requests');
          }
        });
        selectStmt4.execute();
      }
      
      //group invites
      var groupInvitesXML = getXMLValue(loader.data, 'group_invites');
      if(groupInvitesXML) {
        var selectStmt5 = new air.SQLStatement();
        selectStmt5.sqlConnection = connectToDB();
        selectStmt5.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'group_invites' AND notification = '" + groupInvitesXML + "'";
        selectStmt5.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt5.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt5.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('group_invites', '" + groupInvitesXML + "', 1, " + unix_timestamp() + ", 0, " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted group invites');
          }
        });
        selectStmt5.execute();
      }
      
      //event invites
      var eventInvitesXML = getXMLValue(loader.data, 'event_invites');
      if(eventInvitesXML) {
        var selectStmt6 = new air.SQLStatement();
        selectStmt6.sqlConnection = connectToDB();
        selectStmt6.text = "SELECT * FROM notifications WHERE uid = " + readEncryptedStore('uid') + " AND type = 'event_invites' AND notification = '" + eventInvitesXML + "'";
        selectStmt6.addEventListener(air.SQLErrorEvent.ERROR, SQLErrorHandler);
        selectStmt6.addEventListener(air.SQLEvent.RESULT, function(event) {
          var result = selectStmt6.getResult();
          if(result == null || result.data == null) {
            createStmt.text = "INSERT INTO notifications (type, notification, status, inserted, xmltimestamp, uid) VALUES ('event_invites', '" + eventInvitesXML + "', 1, " + unix_timestamp() + ", 0, " + readEncryptedStore('uid') + ")";
            createStmt.execute();
            air.trace('inserted event invites');
          }
        });
        selectStmt6.execute();
      }
    }
  });
};

//getSessionKey
//-----
//[no args]
function getSessionKey() {
  callFacebook({
    method: 'auth.getSession',
    params: {
      'auth_token': readEncryptedStore('authToken')
    },
    onComplete: function(event) {
      var loader = air.URLLoader(event.target);
      
      //error encountered, force login screen
      if(loader.data.indexOf('<error_response') >= 0) {
        //showLogin();
        air.trace('auth.getSession: error response');
        return false;
      }
      
      //air.trace('auth.getSession: ' + loader.data);
      writeEncryptedStore('sessionKey', getXMLValue(loader.data, 'session_key'));
      writeEncryptedStore('uid', getXMLValue(loader.data, 'uid'));
      writeEncryptedStore('secret', getXMLValue(loader.data, 'secret'));
    }
  });
};

//writeEncryptedStore
//-----
//key: string key to store the value as
//value: string to store
function writeEncryptedStore(key, value) {
  var bytes = new air.ByteArray();
  bytes.writeUTFBytes(value);
  air.EncryptedLocalStore.setItem(key, bytes);
};

//readEncryptedStore
//-----
//key: string key to retrieve
function readEncryptedStore(key) {
  var storedValue = air.EncryptedLocalStore.getItem(key);
  if(!storedValue) return false;
  return storedValue.readUTFBytes(storedValue.length);
};

//getXMLValue
//-----
//xml: full xml string
//tagName: xml element tag name
function getXMLValue(xml, tagName) {
  var startIndex = xml.indexOf('<' + tagName + '>');
  if(startIndex >= 0) {
    var endIndex = xml.substring(startIndex).indexOf('</' + tagName + '>');
    if(endIndex >= 0) {
      return xml.substring(startIndex + tagName.length + 2, startIndex + endIndex);
    }
  }
  return false;
};

//unix_timestamp()
//-----
//[no args]
function unix_timestamp() {
  return parseInt((new Date()).getTime() / 1000);
};

//getMonth
//-----
//ts: date object
function getMonth(ts) {
  var months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
  return months[ts.getMonth()];
};

//getDayOfWeek
//-----
//ts: date object
function getDayOfWeek(ts) {
  var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
  return days[ts.getDay()];
};

//clearHistory
//-----
//[no args]
function clearHistory() {
  if(confirm('Clear your notification history?')) {
    var updateStmt = new air.SQLStatement();
    updateStmt.sqlConnection = connectToDB();
    updateStmt.text = "DELETE FROM notifications WHERE uid = " + readEncryptedStore('uid');
    updateStmt.execute();
    closeWindow('history');
  }
};

//exitApp
//-----
//[no args]
function exitApp() {
  air.NativeApplication.nativeApplication.icon.bitmaps = [];
  air.NativeApplication.nativeApplication.exit();
};

//getFriendsDetails
//-----
//id: facebook user id
function getFriendsDetails(id) {
  var selectStmt = new air.SQLStatement();
  selectStmt.sqlConnection = connectToDB();
  selectStmt.text = "SELECT * FROM notifications WHERE id = " + id;
  
  selectStmt.addEventListener(air.SQLEvent.RESULT, function(event) {
    var result = selectStmt.getResult();
    if(result == null || result.data == null) {
      air.trace('invalid friend notification id: ' + id);
    }
    else {
      var row = result.data[0];
      var dp = new DOMParser();
      var xml = dp.parseFromString('<notification>' + row.notification + '</notification>', 'text/xml');
      var uids = xml.documentElement.getElementsByTagName('uid');
      var newContent = '';
      var lastOne = false;
      for(var i = 0; i < uids.length; i++) {
        lastOne = (i+1 == uids.length);
        callFacebook({
          method: 'users.getInfo',
          params: {
            'uids': uids[i].firstChild.nodeValue,
            'fields': 'name,pic_square'
          },
          onComplete: function(event) {
            var loader = air.URLLoader(event.target);
            var imagePath = getXMLValue(loader.data, 'pic_square');
            if(!imagePath || imagePath.length <= 0) imagePath = 'images/no_avatar.png';
            newContent += '<div class="friendItem"><img class="friendAvatar" align="absmiddle" src="' + imagePath + '" />' + getXMLValue(loader.data, 'name') + '</div>';
            //<a href="#" onclick="openProfile(' + getXMLValue(loader.data, 'uid') + ')" class="friendExpand"></a>
            if(lastOne) {
              dijit.byId('friend' + id).setContent(newContent);
            }
          }
        });
      }
    }
  });
  
  selectStmt.execute();
};

//openProfile
//-----
//id: facebook user id
function openProfile(id) {
  air.trace('asdasdasd');
  air.navigateToURL(new air.URLRequest('http://www.facebook.com/profile.php?id=' + id));
};

//appIconClicked
//-----
//[no args]
function appIconClicked() {
  displayHistory();
};